/*
 * ggblocks.h - functions and structures to store a game board ('WORLD') and 
 *      compute and apply moves in it. 
 *
 *    Copyright (C) 2003 Eric Cheung.
 *
 *    This file is part of GG's Blocks.
 *
 *    GG's Blocks is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    GG's Blocks is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with GG's Blocks; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#ifndef _GGBLOCKS_H
#define _GGBLOCKS_H

//  Constants
#define WORLDROW        9
#define WORLDCOL        14
#define NUMDIFFTILES    5
// special tile types
#define EMPTY           -1
#define MIXER           -2


//  Type defines
// a single tile
typedef Int8 TILE;

//  Structures
// stores a gameboard
typedef struct WORLDTag{
    TILE tile[WORLDCOL][WORLDROW];
    Boolean movesLeft;  // whether there are still moves here
    Int16 emptyCol;     // number of columns empty from the left
    Int8 topUpdate[WORLDCOL];       // these specify which area of each column
    Int8 bottomUpdate[WORLDCOL];    // has been modified since the last redraw
} WORLD;
// a (col, row) pair
typedef struct COORDTag {
    Int8 c, r;
} COORD;

//  Function Prototypes
//  marks the entire world for redraw
void worldMarkModified(WORLD *w);

// make a random gameboard 
void worldGenerate(WORLD *w);
// apply a move given the coordinates of the tile the player clicked on.
// this is split into two different functions to allow for animations.
//      To apply a move call one after the other...
//
//          worldApplyMove(w,c,r);
//          worldCompact(w);

// uses a BFS to find all the tiles of the same type surrounding the given 
// coordinates, and replaces them all with EMPTY's
// returns point value of this move if it is legal, -1 otherwise
Int32 worldApplyMove(WORLD *w, Int16 c, Int16 r);

// compacts empty squares out of the gameboard
void worldCompact(WORLD *w);

// computes the final point value of this world
Int32 worldFinalPointValue(WORLD *w);

#endif
